# encoding: utf-8
from calendar import monthrange
from collections import defaultdict
from datetime import date

from efa.lib.report import (
    create_virtual_participation_list,
    filter_date_range,
)
from efa.model.participation import Participation
from euhu.programs.pwe import validate
from pyramid.view import view_config
from ringo.model.base import get_item_list
from ringo.views.helpers import has_role
from ringo.views.home import (
    index_view as ringo_index_view
)
from ringo_bmasreport.reports.durchfuehrungsbericht import recipe
from ringo_news.views import get_news
from ringo_target.lib.detectsystem import (
    get_detectsystem_data,
    detectsystem_detail,
    get_startyear
)
from efa.views.home import get_reminders
from sib.model.participation import SIBParticipation
from sqlalchemy.orm import joinedload


def get_has_errors_items(request):
    """Will return the participations, which has errors and
    should be looked at."""
    participations = request.db.query(Participation) \
        .filter(Participation.has_errors == True) \
        .all()
    items = get_item_list(request, Participation, user=request.user,
                          items=participations).items
    return items


def get_participations_with_start_date(request, clazz, year, month):
    """get participations which start_date is in the month of year."""
    start1 = date(year, month, 1)
    end1 = date(year, month, monthrange(year, month)[1])

    res = request.db.query(clazz).filter(clazz.start_date >= start1) \
        .filter(clazz.start_date <= end1).all()
    return res


def get_participants_for_pns(request, clazz, year, month):
    """Get participations list for relevant participants.
    The get_participations_with_cond results in the interesting
    participations. For this small list of participation get the list."""
    participations = get_participations_with_start_date(request, clazz, year,
                                                        month)
    data = {}
    for participation in participations:
        id1 = participation.participant_id
        if id1 not in data:
            data[id1] = (request.db.query(clazz)
                         .filter(clazz.participant_id == id1).all())

    # Return the participations which is actually a nested list of
    # participations.
    return list(data.values())


CACHE_GET_NEW_ENTRIES = {}


def get_new_entries(request, year, month):
    if not CACHE_GET_NEW_ENTRIES.get((year, month)) is None:
        return CACHE_GET_NEW_ENTRIES[(year, month)]

    start_date = date(year, month, 1)
    end_date = date(year, month, monthrange(year, month)[1])

    if not CACHE_GET_NEW_ENTRIES.get('p'):
        ptns = request.db.query(SIBParticipation) \
            .options(joinedload("participant")) \
            .options(joinedload("owner")) \
            .options(joinedload("sibri")) \
            .options(joinedload("oi")) \
            .options(joinedload("siboi")) \
            .all()
        # Assume that wrong start_date and end_date doesn't hurt here.
        # Else we need an own recipe.
        tmp = defaultdict(list)
        for p in ptns:
            tmp[p.participant_id].append(p)
        participations = create_virtual_participation_list(list(tmp.values()),
                                                           recipe,
                                                           validate,
                                                           "sib")
        CACHE_GET_NEW_ENTRIES['p'] = participations
    participations = CACHE_GET_NEW_ENTRIES.get('p')
    entries = filter_date_range(participations, "start_date", start_date,
                                end_date)
    CACHE_GET_NEW_ENTRIES[(year, month)] = len(entries)
    return len(entries)


@view_config(route_name='home', renderer='/index.mako')
def index_view(request):
    global CACHE_GET_NEW_ENTRIES
    CACHE_GET_NEW_ENTRIES = {}
    values = ringo_index_view(request)
    if request.user:
        values['reminders'] = get_reminders(request)
        values['news'] = get_news(request)
        values['has_errors_items'] = get_has_errors_items(request)
        if request.user and not has_role(request.user, "organisation"):
            if request.registry.settings.get("efa.use_target") == "true":
                values['detectsystem'] = get_detectsystem_data(request,
                                                               get_new_entries)
            else:
                values['detectsystem'] = False
    return values


@view_config(route_name='detectsystem', renderer="/detectsystem/detailed.mako")
def sib_detectsystem(request):
    result_pair = detectsystem_detail(request, get_new_entries)
    data = result_pair[0]
    year = result_pair[1]
    return {"detectsystem": data,
            "startyear": get_startyear(request),
            "endyear": date.today().year,
            "calc_year": year}
